#include <stdio.h>
#include <math.h>
#include <stdlib.h>
#include <iostream>
#include <ostream>
#include <istream>
#include <sstream>
#include <omp.h>
#include "advisor-annotate.h"

using namespace std;

#define NP 100000

double U_new[NP+1];
double U_old[NP+1];
double malha_x[NP+1];
double x , deltaX , velX , tempoFinal , deltaT , tempo , alfa , beta , gama;
double t_ini , t_fim;


long int k , N , contagemTempo;
int myRank , numProcs;

double pulso (double D , double xo , double x) {
	return (exp(-D*((x-xo)*(x-xo))));
}


int main (int argc , char * argv[]){

	N = NP+1;
  	deltaX = (double) 1/(N-1);
	
	for (int i = 0 ; i <= N-1 ; i++) {
		x = (double) i*deltaX;
        	malha_x[i] = x;
        	U_old[i] = pulso (100 , 0.2 , x);
  	}

  	gama = atof(argv[1]);
  	tempoFinal = atof(argv[2]);
 	deltaT = atof(argv[3]);
  	velX = atof(argv[4]);

  	alfa = (gama*deltaT)/(deltaX*deltaX);  

  	k = 0;

	int head = 1;

	//Impressão U_inicial
/*	cout << "U_inicial = [";
	for (int i = 0; i < N-1; i+=10)
		cout << U_old[i] << " ";
	cout << "];" << endl;	
*/


	t_ini = omp_get_wtime();


        #pragma omp parallel
        {

		ANNOTATE_SITE_BEGIN(time_loop);

		while (tempo < tempoFinal){


	      		// Calcula a informacao no pe da caracteristica e atualiza a matriz com esse valor
			#pragma omp for
			for (int i = 1 ; i <= N-2 ; i++) {
				ANNOTATE_ITERATION_TASK(loop_MMOC);
				double deltaXtil = (double) velX*deltaT;
				int delta = floor (deltaXtil/deltaX);
	        		double x1 = (double) (i-delta)*deltaX;
	            		double xo = (double) x1 - deltaX;
	            		double xTil = (double) i*deltaX - deltaXtil;
				if (i - delta - 1 >= 0) {
	            			U_new[i] = ((x1 - xTil)*U_old[i- delta - 1] + (xTil - xo)*U_old[i - delta])/(deltaX);
				} else {
					U_new[i] = U_old[0];
				}
	      		}


			#pragma omp for
	      		for (int i = head+1 ; i <= N-2 ; i+=2) {
				ANNOTATE_ITERATION_TASK(loop_HOP_EXP_1);
		    		U_old[i] = alfa*(U_new[i-1] + U_new[i+1]) + (1 - 2*alfa)*U_new[i];
	      		}
	
			#pragma omp single 
				head = (head+1)%2;				
	
	      		#pragma omp for 
	      		for (int i = head+1; i <= N-2 ; i+=2) {       
				ANNOTATE_ITERATION_TASK(loop_HOP_IMP_1);
		    		U_old[i] = (U_new[i] + alfa*U_old[i-1] + alfa*U_old[i+1])/(1+2*alfa);
	     		}

                        #pragma omp single	
				head = (head+1)%2;

			#pragma omp for
	      		for (int i = head+1 ; i <= N-2 ; i+=2) {
				ANNOTATE_ITERATION_TASK(loop_HOP_EXP_2);
		    		U_new[i] = alfa*(U_old[i-1] + U_old[i+1]) + (1 - 2*alfa)*U_old[i];
	      		}
	
                        #pragma omp single
				head = (head+1)%2;

			#pragma omp for
	      		for (int i = head+1; i <= N-2 ; i+=2) {    
				ANNOTATE_ITERATION_TASK(loop_HOP_IMP_2);   
		    		U_new[i] = (U_old[i] + alfa*U_new[i-1] + alfa*U_new[i+1])/(1+2*alfa);
	     		}

			 #pragma omp single
                                head = (head+1)%2;
	
			#pragma omp for
	      		for (int i = 1; i <= N-1 ; i++) {       
				ANNOTATE_ITERATION_TASK(loop_attribute);
		    		U_old[i] = U_new[i];
	     		}
	
			#pragma omp single
			{
		      		k++;
		                tempo = (double) (k*deltaT);
			}
	  	}

		ANNOTATE_SITE_END();

	}
	
        t_fim = omp_get_wtime();
  
	/* IMPRESSÃO MAPLE */
	cout << "with(plots):" << endl;
  	cout << "pointplot([";
    	for (int i = 0 ; i <= N-1 ; i+=1000) {
	    cout << "[" << malha_x[i] << "," << U_old[i] << "]";
        	if (i != N-1)
	        	cout << ",";
	}
    	cout << "]);" << endl << endl;
 
        cout << "# Tempo = " << t_fim - t_ini << endl;
	cout << "# " << k << " Iteracoes" << endl;
	cout << "# alpha = " << alfa << endl;
        cout << "# gama = " << gama << endl;
        cout << "# Delta T = " << deltaT << endl;
        cout << "# Delta X = " << deltaX << endl;


	
	/* IMPRESSÃO MATLAB 
	cout << "malha_x = [";
	for (int i = 0; i < N-1; i+=1000)
		cout << malha_x[i] << " ";
	cout << "];" << endl;
	
	cout << "U_old = [";
	for (int i = 0; i < N-1; i+=1000)
		cout << U_old[i] << " ";
	cout << "];" << endl;	

	cout << "subplot(1,2,1);plot(malha_x,U_inicial); axis([0 1 0 1]); subplot(1,2,2); plot(malha_x, U_old); axis([0 1 0 1]);";
	
	*/

	return(0);
}
